import math
from gi.repository import Gtk
from delicolour.colour import Colour


class BigColour(Gtk.DrawingArea):
    def __init__(self, border_radius=4, height=75):
        # initial colour is black
        self._colour = Colour.from_rgb(0, 0, 0)

        # save border radius
        self._border_radius = border_radius

        # initialize drawing area
        super().__init__()
        self.set_size_request(0, height)
        self.connect('draw', self._on_draw)

    def _get_size(self):
        alloc = self.get_allocation()

        return alloc.width, alloc.height

    def _on_draw(self, drawing_area, cr):
        def do_path():
            # rectangle with rounded corners
            cr.move_to(br + 2, 2)
            cr.arc(width - br - 2, br + 2, br, -0.5 * math.pi, 0)
            cr.arc(width - br - 2, height - br - 2, br, 0, 0.5 * math.pi)
            cr.arc(br + 2, height - br - 2, br, 0.5 * math.pi, 1 * math.pi)
            cr.arc(br + 2, br + 2, br, 1 * math.pi, -0.5 * math.pi)
            cr.close_path()

        width, height = self._get_size()
        br = self._border_radius

        # set colour
        r, g, b = self._colour.rgb
        r /= 255
        g /= 255
        b /= 255
        cr.set_source_rgb(r, g, b)

        # fill
        do_path()
        cr.fill()

        # stroke
        cr.set_line_width(2)
        cr.set_source_rgb(0.3, 0.3, 0.3)
        do_path()
        cr.stroke()

        return False

    def set_colour(self, colour):
        self._colour = colour
        self.queue_draw()

    def set_rgb(self, r, g, b):
        self._colour.set_rgb(r, g, b)
        self.queue_draw()




import re
from delicolour import config
from gi.repository import Gtk
from gi.repository import Gdk
from gi.repository import Pango


class ScaleEntry(Gtk.Box):
    def __init__(self, label, minval, maxval, page_incr,
                 r=config.TEXT_COLOUR_R, g=config.TEXT_COLOUR_G,
                 b=config.TEXT_COLOUR_B, wrap=False):
        # asked colour
        color = Gdk.Color(red=r * 65535, green=g * 65535, blue=b * 65535)

        # parameters
        self._minval = minval
        self._maxval = maxval
        self._page_incr = page_incr
        self._wrap = wrap
        self._user_on_change = None

        # label
        lbl = Gtk.Label()
        lbl.modify_font(Pango.FontDescription('sans-serif bold 8'))
        lbl.set_width_chars(2)
        lbl.set_text(label)
        lbl.modify_fg(Gtk.StateType.NORMAL, color)
        self._label = label

        # entry
        self._entry = Gtk.Entry()
        self._entry.set_max_length(3)
        self._entry.modify_font(Pango.FontDescription('monospace bold 8'))
        self._entry.set_width_chars(3)
        self._entry.set_max_width_chars(3)
        self._entry.set_alignment(1)
        self._entry.add_events(Gdk.EventMask.SCROLL_MASK | Gdk.EventMask.SMOOTH_SCROLL_MASK)
        self._entry.connect('insert-text', self._on_entry_insert_text)
        self._entry.connect('scroll-event', self._on_entry_scroll_event)
        self._entry_changed_handler = self._entry.connect('changed', self._on_entry_changed)

        # scale
        adjustment = Gtk.Adjustment(0, minval, maxval, 1, page_incr, 0)
        self._scale = Gtk.HScale()
        self._scale.set_adjustment(adjustment)
        self._scale.set_draw_value(False)
        self._scale.set_min_slider_size(15)
        self._scale.connect('change-value', self._on_scale_change_value)
        self._scale.set_can_focus(False)
        self._scale.set_round_digits(0)
        self._scale.set_digits(0)

        # hbox
        super().__init__(spacing=config.MAIN_GUTTER_PX, homogeneous=False)
        self.set_orientation(Gtk.Orientation.HORIZONTAL)
        self.pack_start(lbl, False, False, 0)
        self.pack_start(self._scale, True, True, 0)
        self.pack_start(self._entry, False, False, 0)

    def set_page_incr(self, page_incr):
        self._page_incr = page_incr
        ajd = self._scale.get_adjustment()
        ajd.set_page_increment(page_incr)
        self._scale.set_adjustment(ajd)

    def _do_user_on_change(self):
        if self._user_on_change:
            self._user_on_change()

    def _get_normalized_value(self, value):
        if value is None:
            value = self.value

        value = int(value)

        # clip?
        if value < self._minval:
            return self._minval
        elif value > self._maxval:
            return self._maxval
        elif value is None:
            return self.value
        else:
            return value

    def _set_scale_value_no_emit(self, value):
        self._scale.set_value(value)

    def _set_entry_value_no_emit(self, value):
        txt = str(round(value))
        self._entry.handler_block(self._entry_changed_handler)
        self._entry.set_text(txt)
        self._entry.handler_unblock(self._entry_changed_handler)

    def _set_ctrl_values_no_emit(self, value):
        value = self._get_normalized_value(value)
        self._set_entry_value_no_emit(value)
        self._set_scale_value_no_emit(value)

    def set_value_no_emit(self, value):
        self._set_ctrl_values_no_emit(value)

    @property
    def value(self):
        return self._scale.get_value()

    def _on_entry_insert_text(self, entry, new_text, new_text_length,
                              position):
        if not re.search(r'^[0-9]*$', new_text):
            self._entry.stop_emission('insert-text')

    def _on_entry_changed(self, editable):
        text = self._entry.get_text()

        if len(text) == 0:
            # consider nothing changed
            return

        # value
        next_val = self._get_normalized_value(round(float(text)))

        # set scale value
        self._set_scale_value_no_emit(next_val)

        # notify user
        self._do_user_on_change()

    def _on_entry_scroll_event(self, widget, ev):
        value = self.value
        y_scroll = ev.get_scroll_deltas()[2]

        if y_scroll < 0:
            value -= self._page_incr
        elif y_scroll > 0:
            value += self._page_incr

        # wrap?
        if self._wrap:
            value = (value % (self._maxval + 1 - self._minval)) + self._minval

        # set control values
        self._set_ctrl_values_no_emit(value)

        # notify user
        self._do_user_on_change()

    def _on_scale_change_value(self, scale, scroll, value):
        # value
        value = self._get_normalized_value(value)

        # set scale value
        self._set_scale_value_no_emit(value)

        # set entry value
        self._set_entry_value_no_emit(value)

        # notify user
        self._do_user_on_change()

        return True

    def on_change(self, cb):
        self._user_on_change = cb

    def get_entry(self):
        return self._entry

    def get_scale(self):
        return self._scale

    def set_focus(self):
        self._entry.grab_focus()



import re
from delicolour import config
from gi.repository import Gtk
from gi.repository import Gdk
from gi.repository import Pango


class ColourTextEntry(Gtk.Entry):
    def __init__(self, maxlen):
        # parameters
        self._user_on_change = None
        self._clipboard_sel = Gdk.SELECTION_CLIPBOARD

        # build entry
        super().__init__()
        self.set_max_length(maxlen)
        self.modify_font(Pango.FontDescription('monospace bold 8'))
        self.set_width_chars(maxlen)
        self.set_max_width_chars(maxlen)
        self.connect('insert-text', self._on_insert_text)
        self._changed_handler = self.connect('changed', self._on_changed)
        self.connect('cut-clipboard', self._on_cut)
        self.connect('copy-clipboard', self._on_copy)
        self.connect('paste-clipboard', self._on_paste)

    def _do_user_on_change(self):
        if self._user_on_change:
            self._user_on_change()

    def _on_insert_text(self, entry, new_text, new_text_length,
                        position):
        if not self._match_input(new_text):
            self.stop_emission('insert-text')

    def _on_changed(self, editable):
        if not self._current_text_is_valid():
            # consider nothing changed
            return

        # notify user
        self._do_user_on_change()

    def _set_clipboard(self):
        txt = self._text_to_clipboard()
        Gtk.Clipboard.get(self._clipboard_sel).set_text(txt, -1)

    def _get_clipboard(self):
        txt = Gtk.Clipboard.get(self._clipboard_sel).wait_for_text()

        if txt is not None:
            txt = self._text_from_clipboard(txt)

        return txt

    def _on_cut(self, entry):
        self.stop_emission('cut-clipboard')

    def _on_copy(self, entry):
        if self._current_text_is_valid():
            self._set_clipboard()

        self.stop_emission('copy-clipboard')

    def _on_paste(self, entry):
        text = self._get_clipboard()

        if text is not None:
            self.set_text_no_emit(text)

            # notify user
            self._do_user_on_change()

        self.stop_emission('paste-clipboard')

    @property
    def colour(self):
        raise NotImplementedError()

    def set_colour_no_emit(self, colour):
        raise NotImplementedError()

    def set_text_no_emit(self, text):
        self.handler_block(self._changed_handler)
        self.set_text(self._get_real_text(text))
        self.handler_unblock(self._changed_handler)

    def on_change(self, cb):
        self._user_on_change = cb



import gi


# required GI versions/features
gi.require_version('Gtk', '3.0')


from delicolour import main_window
from gi.repository import Gtk
import delicolour
import argparse
import warnings
import signal


def _parse_args():
    ap = argparse.ArgumentParser()

    ap.add_argument('-V', '--version', action='version',
                    version='%(prog)s v{}'.format(delicolour.__version__))

    # parse args
    args = ap.parse_args()

    return args


def run():
    # enable SIGINT (Ctrl+C)
    signal.signal(signal.SIGINT, signal.SIG_DFL)

    # see <https://bugzilla.gnome.org/show_bug.cgi?id=708676#c4>
    warnings.filterwarnings('ignore', '.*g_value_get_int.*', Warning)

    # main app
    args = _parse_args()
    win = main_window.MainWindow(args)
    win.connect('delete-event', Gtk.main_quit)
    win.show_all()
    Gtk.main()



from delicolour.colour import Colour


class AppModel:
    @staticmethod
    def get_default():
        model = AppModel()
        model.colour = Colour.from_rgb(255, 255, 255)
        model.css_hex_copy_hash = True
        model.css_hex_lower = True
        model.fine_incr = 1

        return model



from delicolour.colour_text_entry import ColourTextEntry
from delicolour.colour import Colour
from gi.repository import Pango
from gi.repository import Gdk
from delicolour import config
from gi.repository import Gtk
import re


class CssRgbEntry(ColourTextEntry):
    def __init__(self):
        super().__init__(18)

    @staticmethod
    def _text_is_valid(text):
        text = text.lower()

        return re.search(r'^rgb\s*\(\s*(\d+)\s*,\s*(\d+)\s*,\s*(\d+)\s*\)$', text)

    def _current_text_is_valid(self):
        return self._text_is_valid(self.get_text())

    def _match_input(self, text):
        text = text.lower()

        return re.search(r'^[ rgb()\,0-9]*$', text)

    def _text_to_clipboard(self):
        return self.get_text().lower()

    def _text_from_clipboard(self, text):
        text = text.lower().strip()

        if self._text_is_valid(text):
            return text

        return None

    def _get_real_text(self, text):
        return text.lower()

    @property
    def colour(self):
        return Colour.from_css_rgb(self.get_text())

    def set_colour_no_emit(self, colour):
        self.set_text_no_emit(colour.get_css_rgb())



import re
from delicolour import config
from delicolour.colour import Colour
from delicolour.colour_text_entry import ColourTextEntry
from gi.repository import Gtk
from gi.repository import Gdk
from gi.repository import Pango


class CssHexEntry(ColourTextEntry):
    def __init__(self, lower=True, copy_hash=True):
        # parameters
        self._lower = lower
        self._copy_hash = copy_hash

        # build parent
        super().__init__(6)

    def _text_len_is_valid(self, text):
        return len(text) in [3, 6]

    def _current_text_is_valid(self):
        text = self.get_text()

        return self._text_len_is_valid(text)

    def _match_input(self, text):
        return re.search(r'^[0-9a-fA-F]*$', text)

    def _text_to_clipboard(self):
        text = self.get_text()

        if self._copy_hash:
            text = '#{}'.format(text)

        return text

    def _text_from_clipboard(self, text):
        text = text.strip()

        if text.startswith('#'):
            text = text[1:]

        if self._text_len_is_valid(text):
            return text

        return None

    def _get_real_text(self, text):
        text = text.upper()

        if self._lower:
            text = text.lower()

        return text

    def set_lower(self, lower):
        self._lower = lower

        if self._current_text_is_valid():
            text = self.get_text().upper()

            if lower:
                text = text.lower()

            self.set_text_no_emit(text)

    def set_copy_hash(self, copy_hash):
        self._copy_hash = copy_hash

    @property
    def colour(self):
        return Colour.from_hex(self.get_text())

    def set_colour_no_emit(self, colour):
        self.set_text_no_emit(colour.hex)
